<?php
	//Chris McCue
	//pilladd.php checks for invalid input and adds pill schedule to database and pill schedule list
	//Modification History:
	//Revision 1:  11/24/2014
	
    include 'medicine-includes.php';

    /*****************************************************************
    ******************************************************************
    **@errorNameAndInvalidDate()
    **
    **@brief performs error checking on empty name and invalid dates
    **
    **@return $errorTally (0 for no errors; >0 for errors)
    **
    ******************************************************************
    *****************************************************************/
    function errorNameAndInvalidDate(){
    
        $errorTally = 0;
    
        //invalid input:  no pill name entered
        if($_POST['pillNameEntered'] == ''){

            $errorTally++;

			//0 for fail, no string to update pill schedule list, invalid input message	
			$pillRetStatus = '0';
            $updatePillStr = "";
            $messageStr = 'Please enter a pill name';
            $returnArr = array('successPillStatus' => $pillRetStatus, 'updatePillList' =>$updatePillStr, 'pillReturnMessage' => $messageStr);
            echo json_encode($returnArr);            
        }

        //put form data into variables
        $errorY = $_POST['pillStartYearEntered'];
        $errorM = $_POST['pillStartMonthEntered'];
        $errorD = $_POST['pillStartDayEntered'];

        //invalid input:  entering 31 for a month with only 30 days
        if(($errorD > 30) && (($errorM == 4) || ($errorM == 6) || ($errorM == 9) || ($errorM == 11))){

			//0 for fail, no string to update pill schedule list, invalid input message	
			$pillRetStatus = '0';
            $updatePillStr = "";
            $messageStr = 'Please enter a valid date. This month only has 30 days.';
            $returnArr = array('successPillStatus' => $pillRetStatus, 'updatePillList' =>$updatePillStr, 'pillReturnMessage' => $messageStr);
            echo json_encode($returnArr);
            
            $errorTally++;
        }

        //invalid input:  February has 29 days at most
        else if(($errorM == 2) && ($errorD > 29)){

			//0 for fail, no string to update pill schedule list, invalid input message	
			$pillRetStatus = '0';
            $updatePillStr = "";
            $messageStr = 'Please enter a valid date. February only has 28-29 days.';
            $returnArr = array('successPillStatus' => $pillRetStatus, 'updatePillList' =>$updatePillStr, 'pillReturnMessage' => $messageStr);
            echo json_encode($returnArr);
            
            $errorTally++;       
        }

        //invalid input:  February has only 28 days on non-leap years
        else if(($errorM == 2) && ($errorY % 4 != 0) && ($errorD == 29)){

			//0 for fail, no string to update pill schedule list, invalid input message			
			$pillRetStatus = '0';
            $updatePillStr = "";
            $messageStr = 'Please enter a valid date. Non-leap years only have 28 days.';
            $returnArr = array('successPillStatus' => $pillRetStatus, 'updatePillList' =>$updatePillStr, 'pillReturnMessage' => $messageStr);
            echo json_encode($returnArr);
            
            $errorTally++;
        }

        return $errorTally;
    }

    /*****************************************************************
    ******************************************************************
    **@calculateTimeStep()
    **
    **@brief calculates time step- (end-start)/(frequency-1) 
    **    
    **@params
    **    $pillEndTime- end time object
    **    $pillStartTime- start time object
    **
    **@return 
    **    $breakFromTimeLoop- for frequency of 1, breaks after 1 iteration
    **    $freqEntered- frequency
    **    $stepHrString- step size hours
    **    $stepMinString- step size minutes
    **
    ******************************************************************
    *****************************************************************/
    function calculateTimeStep($pillEndTime, $pillStartTime){

		$breakFromTimeLoop = 0;
		
        //Find interval between start and end times in minutes
        $pillTimeDiff = date_diff($pillEndTime,$pillStartTime);
        $pillTimeDiffHr = $pillTimeDiff->format("%H");
        $pillTimeDiffMin = $pillTimeDiff->format("%i");
        $pillTimeDiffTotalMin = $pillTimeDiffHr * 60 + $pillTimeDiffMin;

        //Create strings of step size in hours and minutes
        $freqEntered = $_POST['pillFrequencyEntered'];  //FORM, such as 3

        //Corrective for $freqEntered is 1 to avoid divide by zero error
        if($freqEntered == 1){
            $freqEntered++;
            $breakFromTimeLoop = 1;  //breaks from 
        }

        //get hours in step time
        $step = $pillTimeDiffTotalMin/($freqEntered - 1);
        $stepHr = floor($step/60);

        //get minutes in step time. If less than 30 minutes, round to 00.  Else round to 30.
        if(($step%60)<30){
            $stepMin = 0;
        }
        else{
            $stepMin = 30;
        }
		
		//case where there is 0 time step.
		if(($stepMin == 0) && ($stepHr == 0)){
		    $breakFromTimeLoop = 99;
		}
        $stepHrString = $stepHr.' hours';
        $stepMinString = $stepMin.' minutes';

        return array($breakFromTimeLoop, $freqEntered, $stepHrString, $stepMinString);
    }

    /*****************************************************************
    ******************************************************************
    **@generatePillEvents()
    **
    **@brief generates pill schedule and adds to database. 
    **    
    **@params
    **    $username- doctor
    **    $pillStartDateString- start date string
    **    $pillEndDate- end date object
    **    $pillStartTime- start time object
    **    $pillEndTime- end time object
    **    $breakFromTimeLoop- used when frequency is 1 to break out of loop
    **    $stepHrString- step size hours
    **    $stepMinString- step size minutes
    **
    ******************************************************************
    *****************************************************************/
    function generatePillEvents($patientSel, $username, $pillStartDateString, $pillEndDate, $pillStartTime, $pillEndTime, $breakFromTimeLoop, $stepHrString, $stepMinString){

        //start generating dates from start date         
        $pillCurrentDate = date_create($pillStartDateString);

		//iterate through all the dates
        while($pillCurrentDate < $pillEndDate){
            
            //start generating times from start time
            $pillCurrentTime = date_create(date_format($pillStartTime,"H:i"));
            
            //case of frequency of 1
            if($breakFromTimeLoop == 1){           
            
                $pillCurrentDateFormatted = date_format($pillCurrentDate,"m:d:Y");
                $pillCurrentTimeFormatted = date_format($pillCurrentTime,"H:i");
                addMedicine($patientSel, $username, $_POST['pillNameEntered'], $pillCurrentDateFormatted, $pillCurrentTimeFormatted, $_POST['pillDosageEntered']);                       
            }
			
			//all other frequencies
            else{
			
				//generate all the times  
                while($pillCurrentTime <= $pillEndTime){
 
                    $pillCurrentDateFormatted = date_format($pillCurrentDate,"m:d:Y");
                    $pillCurrentTimeFormatted = date_format($pillCurrentTime,"H:i");
                    addMedicine('user', $username, $_POST['pillNameEntered'], $pillCurrentDateFormatted, $pillCurrentTimeFormatted, $_POST['pillDosageEntered']);
                    addMedicine($patientSel, $username, $_POST['pillNameEntered'], $pillCurrentDateFormatted, $pillCurrentTimeFormatted, $_POST['pillDosageEntered']);
					
					//step by the hour and minute step
					date_add($pillCurrentTime,date_interval_create_from_date_string($stepHrString));
                    date_add($pillCurrentTime,date_interval_create_from_date_string($stepMinString));
                }
            }

            //step by one day 
            date_add($pillCurrentDate,date_interval_create_from_date_string('1 day'));    
        }
    }

    /*****************************************************************
    ******************************************************************
    **@calculatePillEndTime()
    **
    **@brief converts form data into end time object. 
    **    
    **@return
    **    $pillEndTime- end time object
    **
    ******************************************************************
    *****************************************************************/
    function calculatePillEndTime(){

		//create date object from form data
        $pillEndTimeString = $_POST['pillEndTimeEntered'];  //FORM, such as '10:00'
        $pillEndTimeSign= $_POST['pillEndSignEntered'];  //FORM, such as 'a.m.'
        $pillEndTime = date_create($pillEndTimeString);
		
		//adjust for military time
		if((($pillEndTimeString == '12:00') || ($pillEndTimeString == '12:30')) && ($pillEndTimeSign == 'a.m.')){
			date_sub($pillEndTime,date_interval_create_from_date_string("12 hours"));
		}
		else if((($pillEndTimeString == '12:00') || ($pillEndTimeString == '12:30')) && ($pillEndTimeSign == 'p.m.')){
			//do nothing
		}
        else if($pillEndTimeSign == 'p.m.'){
            date_add($pillEndTime,date_interval_create_from_date_string("12 hours"));
        }

        return $pillEndTime;
    }

    /*****************************************************************
    ******************************************************************
    **@calculatePillStartTime()
    **
    **@brief converts form data into start time object. 
    **    
    **@return
    **    $pillStartTimeString- end time string needed for later
    **    $pillStartTime- end time object
    **
    ******************************************************************
    *****************************************************************/
    function calculatePillStartTime(){

		//create date object from form data
        $pillStartTimeString = $_POST['pillStartTimeEntered'];  //FORM, like '8:30'
        $pillStartTimeSign = $_POST['pillStartSignEntered'];  //FORM, 'a.m.'
        $pillStartTime = date_create($pillStartTimeString);

		//adjust for military time
		if((($pillStartTimeString == '12:00') || ($pillStartTimeString == '12:30')) && ($pillStartTimeSign == 'a.m.')){
			date_sub($pillStartTime,date_interval_create_from_date_string("12 hours"));
		}
		else if((($pillStartTimeString == '12:00') || ($pillStartTimeString == '12:30')) && ($pillStartTimeSign == 'p.m.')){
			//do nothing
		}		
        else if($pillStartTimeSign == 'p.m.'){
            date_add($pillStartTime,date_interval_create_from_date_string("12 hours"));
        }
    
        return array($pillStartTimeString, $pillStartTime);
    }

    /*****************************************************************
    ******************************************************************
    **@calculatePillStartEndDate()
    **
    **@brief converts form data into start and end date objects. 
    **    
    **@return
    **    $pillStartDateString- start date string needed for later
    **    $pillStartDate-  start date object 
    **    $pillEndDate- end date object
    **
    ******************************************************************
    *****************************************************************/
    function calculatePillStartEndDate(){

		//create date object from form data
        //below from FORM, such as '2014-9-15'      
        $pillStartDateString = $_POST['pillStartYearEntered'].'-'.$_POST['pillStartMonthEntered'].'-'.$_POST['pillStartDayEntered'];
        $pillStartDate = date_create($pillStartDateString);

		//create end time date object from form data
        //FORM, such as '4 days'        
        $pillDuration = $_POST['pillDurationEntered'];  
        $pillEndDate = date_create($pillStartDateString);
        date_add($pillEndDate,date_interval_create_from_date_string($pillDuration));

        return array($pillStartDateString, $pillStartDate, $pillEndDate);
    }

    /**********************************************************
	****************MAIN CODE STARTS HERE**********************
	**********************************************************/
	session_start();
    $username = $_SESSION['user'];
    $patientSel = $_POST['patientToForm'];  

    //if no invalid input, continue execution
    if(errorNameAndInvalidDate() == 0){

        list($pillStartTimeString, $pillStartTime) = calculatePillStartTime();
        $pillEndTime = calculatePillEndTime();

        //invalid input if end time is less than start time
        if($pillEndTime < $pillStartTime){

			//0 for fail, no string to update pill schedules list, invalid input message
			$pillRetStatus = '0';
            $updatePillStr = "";
            $messageStr = 'Please make sure that the start time is less than the end time.';
            $returnArr = array('successPillStatus' => $pillRetStatus, 'updatePillList' =>$updatePillStr, 'pillReturnMessage' => $messageStr);
            echo json_encode($returnArr);                    
        }
    
		//if no invalid input, continue execution
        else{ 
		
            list($breakFromTimeLoop, $freqEntered, $stepHrString, $stepMinString) = calculateTimeStep($pillEndTime, $pillStartTime);
			//invalid input if there is no time step.  will cause infinite loop.
			if($breakFromTimeLoop == 99){
			    //0 for failure, no string to update pill schedules list, invalid input message;
				$pillRetStatus = '0';
                $updatePillStr = "";
                $messageStr = 'The minimum pill schedule interval is 30 minutes.  Please select a larger interval.';
                $returnArr = array('successPillStatus' => $pillRetStatus, 'updatePillList' =>$updatePillStr, 'pillReturnMessage' => $messageStr);
                echo json_encode($returnArr);
			}
			else{
                list($pillStartDateString, $pillStartDate, $pillEndDate) = calculatePillStartEndDate();
                generatePillEvents($patientSel, $username, $pillStartDateString, $pillEndDate, $pillStartTime, $pillEndTime, $breakFromTimeLoop, $stepHrString, $stepMinString);
            
			    //1 for success, pill schedule name to add to pill schedules list, success message
			    $pillRetStatus = '1';
                $updatePillStr = $_POST['pillNameEntered'];
                $messageStr = $_POST['pillNameEntered'].' pill schedule has been successfully added.';
                $returnArr = array('successPillStatus' => $pillRetStatus, 'updatePillList' =>$updatePillStr, 'pillReturnMessage' => $messageStr);
                echo json_encode($returnArr);
			}	
        }
    }
?>


